# Generated by Django 3.2.13 on 2022-08-08 07:47

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('tenancy', '0002_auto_slug'),
        ('nautobot_cable_utils', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CablePlug',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, unique=True)),
                ('name', models.CharField(max_length=50, unique=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='cabletemplate',
            name='tenant',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cable_templates', to='tenancy.tenant'),
        ),
        migrations.AddField(
            model_name='cabletemplate',
            name='plug',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cable_templates', to='nautobot_cable_utils.cableplug'),
        ),
    ]
