# Generated by Django 3.2.15 on 2022-10-07 07:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nautobot_cable_utils', '0004_rename_cable_number_cabletemplate_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cabletemplate',
            name='name',
            field=models.CharField(max_length=100, unique=True),
        ),
    ]
