# Generated by Django 3.1.13 on 2021-11-22 18:39

from django.db import migrations, models
import django.db.models.deletion
import nautobot.utilities.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0005_device_local_context_schema'),
    ]

    operations = [
        migrations.CreateModel(
            name='MeasurementLog',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, unique=True)),
                ('link', models.URLField(blank=True, null=True)),
                ('cable', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='dcim.cable')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CableTemplate',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, unique=True)),
                ('cable_number', models.CharField(max_length=50, unique=True)),
                ('type', models.CharField(blank=True, max_length=50)),
                ('label', models.CharField(blank=True, max_length=100)),
                ('color', nautobot.utilities.fields.ColorField(blank=True, max_length=6)),
                ('length', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('length_unit', models.CharField(blank=True, max_length=50)),
                ('cable', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='dcim.cable')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
